@@ -3,7 +3,7 @@
 """Python link-grammar test script"""
 
 from __future__ import print_function
-import sys, os
+import sys, os, re
 import locale
 import unittest
 
@@ -22,21 +22,24 @@ for v in 'PYTHONPATH', 'srcdir', 'LINK_GRAMMAR_DATA':
     print('{}={}'.format(v, os.environ.get(v)))
 #===
 
-
 from linkgrammar import (Sentence, Linkage, ParseOptions, Link, Dictionary,
                          LG_Error, LG_DictionaryError, LG_TimerExhausted,
                          Clinkgrammar as clg)
 
+print(clg.linkgrammar_get_configuration())
 
 # Show the location and version of the bindings modules
-for module in 'linkgrammar', '_clinkgrammar', 'lg_testutils':
-    if module in sys.modules:
-        print("Using", sys.modules[module], end='')
-        if hasattr(sys.modules[module], '__version__'):
-            print(' version', sys.modules[module].__version__, end='')
-        print()
-    else:
-        print("Warning: Module", module,  "not loaded.")
+for imported_module in 'linkgrammar$', 'clinkgrammar', '_clinkgrammar', 'lg_testutils':
+    module_found = False
+    for module in sys.modules:
+        if re.search(r'^(linkgrammar\.)?'+imported_module, module):
+            print("Using", sys.modules[module], end='')
+            if hasattr(sys.modules[module], '__version__'):
+                print(' version', sys.modules[module].__version__, end='')
+            print()
+            module_found = True
+    if not module_found:
+        print("Warning: Module", imported_module,  "not loaded.")
 
 sys.stdout.flush()
 #===
@@ -85,7 +88,7 @@ class BParseOptionsTestCase(unittest.TestCase):
 
     def test_setting_verbosity_to_not_allow_value_raises_value_error(self):
         po = ParseOptions()
-        self.assertRaises(ValueError, setattr, po, "verbosity", 16)
+        self.assertRaises(ValueError, setattr, po, "verbosity", 121)
 
     def test_setting_verbosity_to_non_integer_raises_type_error(self):
         po = ParseOptions()
@@ -246,12 +249,11 @@ class DBasicParsingTestCase(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
         cls.d = Dictionary()
-        cls.po = None
 
     @classmethod
     def tearDownClass(cls):
         del cls.d
-        del cls.po
+        del cls.parse_sent
 
     def parse_sent(self, text, po=ParseOptions()):
         return list(Sentence(text, self.d, po).parse())
@@ -333,11 +335,10 @@ class DBasicParsingTestCase(unittest.TestCase):
         self.assertEqual(linkage.word(4), '_regex_ive[!].a')
 
     def test_timer_exhausted_exception(self):
-        self.po = ParseOptions(max_parse_time=1)
         self.assertRaises(LG_TimerExhausted,
                           self.parse_sent,
                           "This should take more than one second to parse! " * 20,
-                          self.po)
+                          ParseOptions(max_parse_time=1))
 
 # The tests here are are numbered since their order is important.
 # They depend on the result and state of the previous ones as follows:
@@ -751,6 +752,16 @@ class ZENLangTestCase(unittest.TestCase):
         #            |          |
         #[about] people.p attended.v-d
 
+    def test_2_step_parsing_with_null_links(self):
+        self.po = ParseOptions(min_null_count=0, max_null_count=0)
+
+        sent = Sentence('about people attended', self.d, self.po)
+        linkages = sent.parse()
+        self.assertEqual(len(linkages), 0)
+        self.po = ParseOptions(min_null_count=1, max_null_count=999)
+        linkages = sent.parse(self.po)
+        self.assertEqual(len(linkages), 2)
+        self.assertEqual(linkages.next().unused_word_cost(), 1)
 
 class ZENConstituentsCase(unittest.TestCase):
     @classmethod
