@@ -80,8 +80,8 @@ class ParseOptions(object):
     def verbosity(self, value):
         if not isinstance(value, int):
             raise TypeError("Verbosity must be set to an integer")
-        if value not in range(0,15):
-            raise ValueError("Verbosity levels can be any integer between 0 and 15 inclusive")
+        if value not in range(0,120):
+            raise ValueError("Verbosity levels can be any integer between 0 and 120 inclusive")
         clg.parse_options_set_verbosity(self._obj, value)
 
     @property
@@ -302,7 +302,19 @@ class LG_Error(Exception):
 
     @staticmethod
     def message(msg):
-        return clg._prt_error(msg)
+        """Print a message through the LG error facility"""
+        # Propagate a possible ending "\n" into the format, from which the LG
+        # error facility determine if this is a partial or a complete message.
+        if msg[-1:] == "\n":      # a newline-ended complete message
+            _local_eol = "\n"
+            msg = msg[:-1]
+        elif msg[-2:] == "\n\\":  # a newline-ended partial message
+            _local_eol = ""
+            msg = msg[:-1]
+        else:
+            _local_eol = ""       # a partial message
+
+        return clg._prt_error('%s'+_local_eol, msg)
 
     @staticmethod
     def _default_handler(errinfo, data):
@@ -493,11 +505,12 @@ class Sentence(object):
         return clg.sentence_null_count(self._obj)
 
     class sentence_parse(object):
-        def __init__(self, sent):
+        def __init__(self, sent, parse_options):
             self.sent = sent
             self.num = 0
-            self.rc = clg.sentence_parse(sent._obj, sent.parse_options._obj)
-            if clg.parse_options_timer_expired(sent.parse_options._obj):
+            self.parse_options = sent.parse_options if parse_options is None else parse_options
+            self.rc = clg.sentence_parse(sent._obj, self.parse_options._obj)
+            if clg.parse_options_timer_expired(self.parse_options._obj):
                 raise LG_TimerExhausted()
 
         def __nonzero__(self):
@@ -517,7 +530,7 @@ class Sentence(object):
         def next(self):
             if self.num == clg.sentence_num_valid_linkages(self.sent._obj):
                 raise StopIteration()
-            linkage = Linkage(self.num, self.sent, self.sent.parse_options._obj)
+            linkage = Linkage(self.num, self.sent, self.parse_options._obj)
             if not linkage:  # SAT sentinel value
                 raise StopIteration()
             self.num += 1
@@ -525,5 +538,5 @@ class Sentence(object):
 
         __next__ = next             # Account python3
 
-    def parse(self):
-        return self.sentence_parse(self)
+    def parse(self, parse_options=None):
+        return self.sentence_parse(self, parse_options)
