@@ -103,22 +103,24 @@ static int condesc_by_uc_num(const void *a, const void *b)
 
 #define LENGTH_LINIT_WILD_TYPE WILD_TYPE
 
+/**
+ * Set the length limit of all the connectors that match those in e.
+ * XXX A connector in e that doesn't match any other connector cannot
+ * be detected, because it has been inserted into the connector table and
+ * hence matches at least itself.
+ */
 static void set_condesc_length_limit(Dictionary dict, const Exp *e, int length_limit)
 {
 	size_t exp_num_con;
 	ConTable *ct = &dict->contable;
 	condesc_t **sdesc = ct->sdesc;
-	condesc_t **econlist = NULL;
+	condesc_t **econlist;
 
-	if (e)
-	{
-		/* Create a connector list from the given expression. */
-		exp_num_con = get_connectors_from_expression(NULL, e);
-		econlist = alloca(exp_num_con * sizeof(*econlist));
-		get_connectors_from_expression(econlist, e);
-	}
-
-	if ((NULL == econlist) || (NULL == econlist[0])) return;
+	/* Create a connector list from the given expression. */
+	exp_num_con = get_connectors_from_expression(NULL, e);
+	if (0 == exp_num_con) return; /* Empty connector list. */
+	econlist = alloca(exp_num_con * sizeof(*econlist));
+	get_connectors_from_expression(econlist, e);
 
 	qsort(econlist, exp_num_con, sizeof(*econlist), condesc_by_uc_num);
 
@@ -133,6 +135,7 @@ static void set_condesc_length_limit(Dictionary dict, const Exp *e, int length_l
 
 		for (; en < exp_num_con; en++)
 			if (econlist[en]->uc_num >= sdesc[cn]->uc_num) break;
+		if (en == exp_num_con) break;
 
 		if (econlist[en]->uc_num != sdesc[cn]->uc_num) continue;
 		restart_cn = cn+1;
@@ -162,6 +165,18 @@ static void set_condesc_length_limit(Dictionary dict, const Exp *e, int length_l
 	}
 }
 
+static void condesc_length_limit_def_delete(ConTable *ct)
+{
+	length_limit_def_t *l_next;
+
+	for (length_limit_def_t *l = ct->length_limit_def; NULL != l; l = l_next)
+	{
+		l_next = l->next;
+		free(l);
+	}
+	ct->length_limit_def = NULL;
+}
+
 void set_all_condesc_length_limit(Dictionary dict)
 {
 	ConTable *ct = &dict->contable;
@@ -186,23 +201,18 @@ void set_all_condesc_length_limit(Dictionary dict)
 		}
 	}
 
-	length_limit_def_t *l_next;
-	for (length_limit_def_t *l = ct->length_limit_def; NULL != l; l = l_next)
-	{
-		l_next = l->next;
-		free(l);
-	}
+	condesc_length_limit_def_delete(&dict->contable);
 
 	if (verbosity_level(D_SPEC+1))
 	{
-		printf("%5s %-6s %3s\n", "num", "uc_num", "ll");
+		prt_error("Debug:\n%5s %-6s %3s\n\\", "num", "uc_num", "ll");
 		for (size_t n = 0; n < ct->num_con; n++)
 		{
-			printf("%5zu %6d %3d %s\n", n, ct->sdesc[n]->uc_num,
+			prt_error("%5zu %6d %3d %s\n\\", n, ct->sdesc[n]->uc_num,
 			       ct->sdesc[n]->length_limit, ct->sdesc[n]->string);
 		}
+		prt_error("\n");
 	}
-
 }
 
 /* ======================================================== */
@@ -419,6 +429,7 @@ void condesc_delete(Dictionary dict)
 	pool_delete(dict->contable.mempool);
 	free(dict->contable.hdesc);
 	free(dict->contable.sdesc);
+	condesc_length_limit_def_delete(&dict->contable);
 }
 
 static condesc_t **condesc_find(ConTable *ct, const char *constring, int hash)
