@@ -15,20 +15,17 @@
 #include "dict-common/dict-affix.h"
 #include "dict-common/dict-api.h"
 #include "dict-common/dict-common.h"
-#include "dict-common/dict-defines.h"  // For LEFT_WORD
 #include "dict-common/dict-impl.h"
 #include "dict-common/dict-utils.h"
 #include "dict-common/file-utils.h"
 #include "dict-common/idiom.h"
+#include "dict-common/regex-morph.h"
 #include "post-process/pp_knowledge.h"
 #include "read-dict.h"
 #include "read-regex.h"
-#include "dict-common/regex-morph.h"
+#include "string-set.h"
 #include "tokenize/anysplit.h"        // Initialize anysplit here ...
 #include "tokenize/spellcheck.h"      // Initialize spellcheck here ...
-#include "string-set.h"
-#include "dict-sql/read-sql.h"  /* Temporary hack */
-
 
 /***************************************************************
 *
@@ -307,10 +304,15 @@ Dictionary dictionary_create_from_file(const char * lang)
 }
 
 
+#ifdef USE_VITERBI
 /**
  * Use "string" as the input dictionary. All of the other parts,
  * including post-processing, affix table, etc, are NULL.
  * This routine is intended for unit-testing ONLY.
+ *
+ * FIXME? (Used only by code that is in use for now.)
+ * 1. get_default_locale() returns locale, not language.
+ * 2. "lang" memory leak on success.
  */
 Dictionary dictionary_create_from_utf8(const char * input)
 {
@@ -332,4 +334,5 @@ Dictionary dictionary_create_from_utf8(const char * input)
 
 	return dictionary;
 }
+#endif // USE_VITERBI
 
