@@ -13,7 +13,7 @@
 #ifndef _LG_READ_DICT_H_
 #define  _LG_READ_DICT_H_
 
-#include "dict-structures.h"
+#include "dict-common/dict-structures.h"
 
 void print_dictionary_data(Dictionary dict);
 void print_dictionary_words(Dictionary dict);
@@ -21,9 +21,11 @@ void print_dictionary_words(Dictionary dict);
 Dictionary dictionary_create_from_file(const char * lang);
 bool read_dictionary(Dictionary dict);
 
-Dict_node * lookup_list(const Dictionary dict, const char *s);
-bool boolean_lookup(Dictionary dict, const char *s);
-void free_lookup(Dict_node *llist);
+Dict_node * file_lookup_list(const Dictionary dict, const char *s);
+Dict_node * file_lookup_wild(Dictionary dict, const char *s);
+bool file_boolean_lookup(Dictionary dict, const char *s);
+void file_free_lookup(Dict_node *llist);
+
 void free_insert_list(Dict_node *ilist);
 void insert_list(Dictionary dict, Dict_node * p, int l);
 
