@@ -14,6 +14,7 @@
 #include "api-structures.h"
 #include "connectors.h"
 #include "disjunct-utils.h"
+#include "dict-common/dict-common.h"     // For contable
 #include "post-process/post-process.h"
 #include "post-process/pp-structures.h"
 #include "print/print.h"  // For print_disjunct_counts()
@@ -30,12 +31,10 @@
 //#define ALT_DISJUNCT_CONSISTENCY
 
 #if defined(ALT_MUTUAL_CONSISTENCY) || defined(ALT_DISJUNCT_CONSISTENCY)
-#include "tokenize/tok-structures.h"
 #endif /* ALT_MUTUAL_CONSISTENCY || ALT_DISJUNCT_CONSISTENCY */
 
 #define D_PRUNE 5
 
-#define CONTABSZ 8192
 typedef Connector * connector_table;
 
 /* Indicator that this connector cannot be used -- that its "obsolete".  */
@@ -57,6 +56,7 @@ struct power_table_s
 	unsigned int *r_table_size;
 	C_list *** l_table;
 	C_list *** r_table;
+	Pool_desc *memory_pool;
 };
 
 typedef struct cms_struct Cms;
@@ -181,35 +181,15 @@ struct prune_context_s
    deletable, this is equivalent to RUTHLESS.   --DS, 7/97
 */
 
-static void free_C_list(C_list * t)
-{
-	C_list *xt;
-	for (; t!=NULL; t=xt) {
-		xt = t->next;
-		xfree((char *)t, sizeof(C_list));
-	}
-}
-
 /**
  * free all of the hash tables and C_lists
  */
 static void power_table_delete(power_table *pt)
 {
-	unsigned int w;
-	unsigned int i;
-
-	for (w = 0; w < pt->power_table_size; w++)
+	pool_delete(pt->memory_pool);
+	for (WordIdx w = 0; w < pt->power_table_size; w++)
 	{
-		for (i = 0; i < pt->l_table_size[w]; i++)
-		{
-			free_C_list(pt->l_table[w][i]);
-		}
 		xfree((char *)pt->l_table[w], pt->l_table_size[w] * sizeof (C_list *));
-
-		for (i = 0; i < pt->r_table_size[w]; i++)
-		{
-			free_C_list(pt->r_table[w][i]);
-		}
 		xfree((char *)pt->r_table[w], pt->r_table_size[w] * sizeof (C_list *));
 	}
 	xfree(pt->l_table_size, 2 * pt->power_table_size * sizeof(unsigned int));
@@ -221,12 +201,11 @@ static void power_table_delete(power_table *pt)
  * The disjunct d (whose left or right pointer points to c) is put
  * into the appropriate hash table
  */
-static void put_into_power_table(unsigned int size, C_list ** t, Connector * c, bool shal)
+static void put_into_power_table(C_list * m, unsigned int size, C_list ** t,
+                                 Connector * c, bool shal)
 {
 	unsigned int h;
-	C_list * m;
-	h = connector_hash(c) & (size-1);
-	m = (C_list *) xalloc (sizeof(C_list));
+	h = connector_uc_num(c) & (size-1);
 	m->next = t[h];
 	t[h] = m;
 	m->c = c;
@@ -244,6 +223,8 @@ static power_table * power_table_new(Sentence sent)
 	C_list ** t;
 	Disjunct * d;
 	Connector * c;
+#define TOPSZ 32768
+	size_t lr_table_max_usage = MIN(sent->dict->contable.num_con, TOPSZ);
 
 	pt = (power_table *) xalloc (sizeof(power_table));
 	pt->power_table_size = sent->length;
@@ -252,6 +233,10 @@ static power_table * power_table_new(Sentence sent)
 	pt->l_table = xalloc (2 * sent->length * sizeof(C_list **));
 	pt->r_table = pt->l_table + sent->length;
 
+	Pool_desc *mp = pt->memory_pool = pool_new(__func__, "C_list",
+	                   /*num_elements*/2048, sizeof(C_list),
+	                   /*zero_out*/false, /*align*/false, /*exact*/false);
+
 	for (w=0; w<sent->length; w++)
 	{
 		/* The below uses variable-sized hash tables. This seems to
@@ -270,9 +255,7 @@ static power_table * power_table_new(Sentence sent)
 		 * Strong dependence on the hashing algo!
 		 */
 		len = left_connector_count(sent->word[w].d);
-		size = next_power_of_two_up(len);
-#define TOPSZ 32768
-		if (TOPSZ < size) size = TOPSZ;
+		size = next_power_of_two_up(MIN(len, lr_table_max_usage));
 		pt->l_table_size[w] = size;
 		t = pt->l_table[w] = (C_list **) xalloc(size * sizeof(C_list *));
 		for (i=0; i<size; i++) t[i] = NULL;
@@ -280,16 +263,15 @@ static power_table * power_table_new(Sentence sent)
 		for (d=sent->word[w].d; d!=NULL; d=d->next) {
 			c = d->left;
 			if (c != NULL) {
-				put_into_power_table(size, t, c, true);
+				put_into_power_table(pool_alloc(mp), size, t, c, true);
 				for (c=c->next; c!=NULL; c=c->next) {
-					put_into_power_table(size, t, c, false);
+					put_into_power_table(pool_alloc(mp), size, t, c, false);
 				}
 			}
 		}
 
 		len = right_connector_count(sent->word[w].d);
-		size = next_power_of_two_up(len);
-		if (TOPSZ < size) size = TOPSZ;
+		size = next_power_of_two_up(MIN(len, lr_table_max_usage));
 		pt->r_table_size[w] = size;
 		t = pt->r_table[w] = (C_list **) xalloc(size * sizeof(C_list *));
 		for (i=0; i<size; i++) t[i] = NULL;
@@ -297,9 +279,9 @@ static power_table * power_table_new(Sentence sent)
 		for (d=sent->word[w].d; d!=NULL; d=d->next) {
 			c = d->right;
 			if (c != NULL) {
-				put_into_power_table(size, t, c, true);
+				put_into_power_table(pool_alloc(mp), size, t, c, true);
 				for (c=c->next; c!=NULL; c=c->next){
-					put_into_power_table(size, t, c, false);
+					put_into_power_table(pool_alloc(mp), size, t, c, false);
 				}
 			}
 		}
@@ -324,8 +306,6 @@ static void clean_table(unsigned int size, C_list ** t)
 			if (m->c->nearest_word != BAD_WORD) {
 				m->next = head;
 				head = m;
-			} else {
-				xfree((char *) m, sizeof(C_list));
 			}
 		}
 		t[i] = head;
@@ -441,6 +421,7 @@ static bool possible_connection(prune_context *pc,
 {
 	int dist;
 	if ((!lshallow) && (!rshallow)) return false;
+	if (!easy_match_desc(lc->desc, rc->desc)) return false;
 
 	/* Two deep connectors can't work */
 	if ((lc->nearest_word > rword) || (rc->nearest_word < lword)) return false;
@@ -478,7 +459,7 @@ static bool possible_connection(prune_context *pc,
 	if (!alt_consistency(pc, lc, rc, lword, rword, lr)) return false;
 #endif
 
-	return easy_match(lc->string, rc->string);
+	return true;
 }
 
 /**
@@ -494,7 +475,7 @@ right_table_search(prune_context *pc, int w, Connector *c,
 	power_table *pt = pc->pt;
 
 	size = pt->r_table_size[w];
-	h = connector_hash(c) & (size-1);
+	h = connector_uc_num(c) & (size-1);
 	for (cl = pt->r_table[w][h]; cl != NULL; cl = cl->next)
 	{
 		if (possible_connection(pc, cl->c, c, cl->shallow, shallow, w, word_c, true))
@@ -516,7 +497,7 @@ left_table_search(prune_context *pc, int w, Connector *c,
 	power_table *pt = pc->pt;
 
 	size = pt->l_table_size[w];
-	h = connector_hash(c) & (size-1);
+	h = connector_uc_num(c) & (size-1);
 	for (cl = pt->l_table[w][h]; cl != NULL; cl = cl->next)
 	{
 		if (possible_connection(pc, c, cl->c, shallow, cl->shallow, word_c, w, false))
@@ -1008,7 +989,7 @@ static int pp_prune(Sentence sent, Parse_Options opts)
 				Connector *c;
 				for (c = ((dir) ? (d->left) : (d->right)); c != NULL; c = c->next)
 				{
-					insert_in_cms_table(cmt, c->string);
+					insert_in_cms_table(cmt, connector_string(c));
 				}
 			}
 		}
@@ -1042,7 +1023,7 @@ static int pp_prune(Sentence sent, Parse_Options opts)
 
 							if (strchr(selector, '*') != NULL) continue;  /* If it has a * forget it */
 
-							if (!post_process_match(selector, c->string)) continue;
+							if (!post_process_match(selector, connector_string(c))) continue;
 
 							/*
 							printf("pp_prune: trigger ok.  selector = %s  c->string = %s\n", selector, c->string);
@@ -1073,7 +1054,7 @@ static int pp_prune(Sentence sent, Parse_Options opts)
 						Connector *c;
 						for (c = ((dir) ? (d->left) : (d->right)); c != NULL; c = c->next)
 						{
-							change |= delete_from_cms_table(cmt, c->string);
+							change |= delete_from_cms_table(cmt, connector_string(c));
 						}
 					}
 				}
