@@ -70,16 +70,18 @@ struct CNode_s
 
 /* ================================================================ */
 
+/**
+ * @param s A Label
+ * @param t A Label
+ * @return True if the uppercase parts of s and t are equal, else false.
+ */
 static bool uppercompare(const char * s, const char * t)
 {
-#if 0  /* Non-ASCII definition are not supported.  */
-	return (false == utf8_upper_match(s,t));
-#endif
 	while (isupper(*s) || isupper(*t))
 	{
-		if (*s++ != *t++) return true;
+		if (*s++ != *t++) return false;
 	}
-	return false;
+	return true;
 }
 
 /**
@@ -411,17 +413,17 @@ static void generate_misc_word_info(con_context_t * ctxt, Linkage linkage)
 	for (l1 = 0; l1 < linkage_get_num_links(linkage); l1++) {
 		w1=linkage_get_link_rword(linkage, l1);
 		label1 = linkage_get_link_label(linkage, l1);
-		if ((uppercompare(label1, "S")==0) ||
-			(uppercompare(label1, "SX")==0) ||
-			(uppercompare(label1, "SF")==0)) {
+		if ((uppercompare(label1, "S")) ||
+			(uppercompare(label1, "SX")) ||
+			(uppercompare(label1, "SF"))) {
 			ctxt->wordtype[w1] = STYPE;
 			for (l2 = 0; l2 < linkage_get_num_links(linkage); l2++) {
 				w2=linkage_get_link_lword(linkage, l2);
 				label2 = linkage_get_link_label(linkage, l2);
 				if ((w1 == w2) &&
 					((post_process_match("Pg#b", label2)==1) ||
-					 (uppercompare(label2, "I")==0) ||
-					 (uppercompare(label2, "PP")==0) ||
+					 (uppercompare(label2, "I")) ||
+					 (uppercompare(label2, "PP")) ||
 					 (post_process_match("Pv", label2)==1))) {
 					/* Pvf, Pgf? */
 					ctxt->wordtype[w1] = PTYPE;
@@ -469,15 +471,15 @@ static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_to
 		   the paraphrasing clause doesn't get
 		   an S. (This is true in Treebank II, not Treebank I) */
 
-		if (uppercompare(ctxt->constituent[c].start_link, "CP") == 0)
+		if (uppercompare(ctxt->constituent[c].start_link, "CP"))
 		{
 			ctxt->constituent[c].valid = false;
 		}
 
 		/* If it's a possessive with an "'s", the NP on the left
 		   should be extended to include the "'s". */
-		if ((uppercompare(ctxt->constituent[c].start_link, "YS") == 0) ||
-			(uppercompare(ctxt->constituent[c].start_link, "YP") == 0))
+		if ((uppercompare(ctxt->constituent[c].start_link, "YS")) ||
+			(uppercompare(ctxt->constituent[c].start_link, "YP")))
 		{
 			ctxt->constituent[c].right++;
 		}
@@ -500,7 +502,7 @@ static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_to
 		}
 
 		/* If the constituent is an S started by "but" or "and" at
-		   the beginning of the sentence, it should be ignored. */
+		   the beginning of the 67411142sentence, it should be ignored. */
 
 		if ((strcmp(ctxt->constituent[c].start_link, "Wdc") == 0) &&
 			(ctxt->constituent[c].left == 2))
@@ -1106,15 +1108,15 @@ static char * print_flat_constituents(Linkage linkage)
 	size_t wts = linkage->num_words * sizeof(WType);
 	size_t cns = (linkage->num_links + linkage->num_words) * sizeof(constituent_t);
 
-	con_context_t *ctxt = (con_context_t *) alloca(sizeof(con_context_t));
-	memset(ctxt, 0, sizeof(con_context_t));
-	ctxt->wordtype = (WType *) alloca(wts);
-	memset(ctxt->wordtype, 0, wts);
-	ctxt->conlen = linkage->num_links + linkage->num_words;
-	ctxt->constituent = (constituent_t *) alloca(cns);
-	memset(ctxt->constituent, 0, cns);
+	con_context_t ctxt;
+	memset(&ctxt, 0, sizeof(con_context_t));
+	ctxt.wordtype = (WType *) alloca(wts);
+	memset(ctxt.wordtype, 0, wts);
+	ctxt.conlen = linkage->num_links + linkage->num_words;
+	ctxt.constituent = (constituent_t *) alloca(cns);
+	memset(ctxt.constituent, 0, cns);
 
-	return do_print_flat_constituents(ctxt, linkage);
+	return do_print_flat_constituents(&ctxt, linkage);
 }
 
 static CType token_type (char *token)
@@ -1129,9 +1131,8 @@ static CType token_type (char *token)
 static CNode * make_CNode(char *q)
 {
 	CNode * cn;
-	cn = (CNode *) exalloc(sizeof(CNode));
-	cn->label = (char *) exalloc(sizeof(char)*(strlen(q)+1));
-	strcpy(cn->label, q);
+	cn = (CNode *) malloc(sizeof(CNode));
+	cn->label = strdup(q);
 	cn->child = cn->next = (CNode *) NULL;
 	cn->next = (CNode *) NULL;
 	cn->start = cn->end = -1;
@@ -1264,18 +1265,16 @@ static int assign_spans(CNode * n, int start)
 static CNode * linkage_constituent_tree(Linkage linkage)
 {
 	char *p, *q, *saveptr;
-	int len;
 	CNode * root;
 
 	p = print_flat_constituents(linkage);
 
-	len = strlen(p);
 	q = strtok_r(p, " ", &saveptr);
 	assert(token_type(q) == OPEN_TOK, "Illegal beginning of string");
 	root = make_CNode(q+1);
 	root = parse_string(root, &saveptr);
 	assign_spans(root, 0);
-	exfree(p, sizeof(char)*(len+1));
+	free(p);
 	return root;
 }
 
@@ -1292,8 +1291,8 @@ static void linkage_free_constituent_tree(CNode * n)
 		x=m->next;
 		linkage_free_constituent_tree(m);
 	}
-	exfree(n->label, sizeof(char)*(strlen(n->label)+1));
-	exfree(n, sizeof(CNode));
+	free(n->label);
+	free(n);
 }
 
 /**
@@ -1332,5 +1331,5 @@ char * linkage_print_constituent_tree(Linkage linkage, ConstituentDisplayStyle m
 
 void linkage_free_constituent_tree_str(char * s)
 {
-	exfree(s, strlen(s)+1);
+	free(s);
 }
