@@ -17,6 +17,7 @@
 #include "dict-common/dict-utils.h"   // For size_of_expression()
 #include "disjunct-utils.h"
 #include "linkage/linkage.h"
+#include "post-process/post-process.h" // for compute_domain_names()
 #include "print.h"
 #include "tokenize/word-structures.h" // For Word_struct
 #include "wcwidth.h"
@@ -30,14 +31,24 @@
 #define DEPT_CHR ('d') /* Single char marking dependent word */
 
 /**
- * Find the position of the center of each word.
- * Also find the offset of each word, relative to the previous one,
- * needed to fully fit the names of the links between them.
+ * Find the position, measured in column-widths, of the center of
+ * each word.  Also find the offset of each word, relative to the
+ * previous one, needed to fully fit the names of the links between
+ * them. The offset is again measured in column-widths, not in bytes!
+ *
+ * By "column widths", it is meant the number of terminal columns
+ * needed to display a glyph. Almost all indoeuropean glyphs are one
+ * column wide. Almost all CJK glyphs (hanzi, kanji) are two columns
+ * wide. These widths are NOT to be confuxed with UTF-8 byte size.
+ *
  * FIXME Long link names between more distant words may still not
  * fit the space between these words.
  *
- * Return the number of characters needed for the all the words,
- * including the space needed for the link names as described above.
+ * Return the number of bytes needed for the all the words, including
+ * the space needed for the link names as described above.  Note that
+ * this byte length might be less than the glyph width! e.g. the
+ * ASCII chars in the rangel 01 to 1F usually print in two columns,
+ * but require only one byte to encode.
  */
 static size_t
 set_centers(const Linkage linkage, int center[], int word_offset[],
@@ -47,7 +58,7 @@ set_centers(const Linkage linkage, int center[], int word_offset[],
 	size_t n;
 	int start_word = print_word_0 ? 0 : 1;
 	int *link_len = alloca(linkage->num_words * sizeof(*link_len));
-	size_t max_line_len = 0; /* Needed picture array line length */
+	size_t max_bytes_in_line = 0; /* Needed picture array line length */
 
 	memset(link_len, 0, linkage->num_words * sizeof(*link_len));
 
@@ -58,34 +69,40 @@ set_centers(const Linkage linkage, int center[], int word_offset[],
 		if ((l->lw + 1 == l->rw) && (NULL != l->link_name))
 		{
 			link_len[l->rw] = strlen(l->link_name) +
-				(DEPT_CHR == l->rc->string[0]) +
-				(HEAD_CHR == l->rc->string[0]) +
-				(DEPT_CHR == l->lc->string[0]) +
-				(HEAD_CHR == l->lc->string[0]);
+				(DEPT_CHR == connector_string(l->rc)[0]) +
+				(HEAD_CHR == connector_string(l->rc)[0]) +
+				(DEPT_CHR == connector_string(l->lc)[0]) +
+				(HEAD_CHR == connector_string(l->lc)[0]);
 		}
 	}
 
 	tot = 0;
 	for (i = start_word; i < N_words_to_print; i++)
 	{
-		int len, center_t;
+		int len, middle;
 
 		/* Centers obtained by counting the characters column widths,
 		 * not the bytes in the string. */
 		len = utf8_strwidth(linkage->word[i]);
-		center_t = tot + (len/2);
+
+		middle = tot + (len/2);
 #if 1 /* Long labels - disable in order to compare output with old versions. */
 		if (i > start_word)
-			center[i] = MAX(center_t, center[i-1] + link_len[i] + 1);
+			center[i] = MAX(middle, center[i-1] + link_len[i] + 1);
 		else
 #endif
-			center[i] = center_t;
-		word_offset[i] = center[i] - center_t;
+			center[i] = middle;
+		word_offset[i] = center[i] - middle;
 		tot += len+1 + word_offset[i];
-		max_line_len += word_offset[i] + strlen(linkage->word[i]) + 1;
+
+		// We use 2x strlen, because invalid UTF-8 chars get padding.
+		// But we don't know how much padding, right now.  In general,
+		// this will sharply over-count, but is needed to deal with
+		// garbage data that is attempting to crash/buffer-overflow.
+		max_bytes_in_line += word_offset[i] + 2*strlen(linkage->word[i]) + 1;
 	}
 
-	return max_line_len;
+	return max_bytes_in_line;
 }
 
 /* The following are all for generating postscript */
@@ -100,24 +117,38 @@ typedef struct
 
 /**
  * Prints s then prints the last |t|-|s| characters of t.
- * if s is longer than t, it truncates s.
+ * If s is longer than t, it truncates s.
  * Handles utf8 strings correctly.
+ * Assumes that the characters of t are all blanks,
+ * so that column-width of t == num bytes in t == num chars in t.
  */
 static void left_append_string(dyn_str * string, const char * s, const char * t)
 {
-	size_t i;
-	size_t slen = utf8_strwidth(s);
-	size_t tlen = utf8_strwidth(t);
+	/* These are column-widths. */
+	size_t twidth = strlen(t);
+	size_t swidth = utf8_strwidth(s);
 
-	for (i = 0; i < tlen; i++)
+	for (size_t i = 0; i < twidth; )
 	{
-		if (i < slen)
-			append_utf8_char(string, s);
-		else
-			append_utf8_char(string, t);
+		if (i < swidth && *s != '\0')
+		{
+			// The width might be negative, if the value is not a valid
+			// UTF-8 character. Assume that it will be printed with a
+			// two-column-wide "box font".
+			int sw = utf8_charwidth(s);
+			if (sw < 0) sw = 2;
+			i += sw;
+			t += sw;
 
-		s += utf8_charlen(s);
-		t += utf8_charlen(t);
+			// Meanwhile, s advances by bytes, and not by column widths.
+			s += append_utf8_char(string, s);
+		}
+		else
+		{
+			append_utf8_char(string, t);
+			i++;
+			t++;
+		}
 	}
 }
 
@@ -179,6 +210,8 @@ char * linkage_print_links_and_domains(const Linkage linkage)
 	dyn_str * s = dyn_str_new();
 	const char ** dname;
 
+	compute_domain_names(linkage);
+
 	longest = 0;
 	for (link=0; link<N_links; link++)
 	{
@@ -264,7 +297,7 @@ char * linkage_print_disjuncts(const Linkage linkage)
 		if (NULL == disj) continue;
 
 		/* Subscript mark will be cleaned up by append_string(). */
-		infword = disj->word_string;
+		// infword = disj->word_string;
 
 		/* ... except that subscript marks confuse utf8_strwidth() */
 		/* linkage->word[w] should be same as disj->word_string except
@@ -323,7 +356,7 @@ build_linkage_postscript_string(const Linkage linkage,
 			if (ppla[j].lw == 0) {
 				if (ppla[j].rw == linkage->num_words-1) continue;
 				N_wall_connectors ++;
-				if (easy_match(ppla[j].lc->string, LEFT_WALL_SUPPRESS)) {
+				if (easy_match(connector_string(ppla[j].lc), LEFT_WALL_SUPPRESS)) {
 					suppressor_used = true;
 				}
 			}
@@ -339,7 +372,7 @@ build_linkage_postscript_string(const Linkage linkage,
 		for (j=0; j<N_links; j++) {
 			if (ppla[j].rw == linkage->num_words-1) {
 				N_wall_connectors ++;
-				if (easy_match(ppla[j].lc->string, RIGHT_WALL_SUPPRESS)) {
+				if (easy_match(connector_string(ppla[j].lc), RIGHT_WALL_SUPPRESS)) {
 					suppressor_used = true;
 				}
 			}
@@ -395,26 +428,25 @@ build_linkage_postscript_string(const Linkage linkage,
 
 static void diagram_alloc_tmpmem(size_t **start, char ***pic, char ***xpic,
                                  size_t *cur_height, size_t max_height,
-                                 size_t max_line, size_t line_len)
+                                 size_t max_bytes, size_t num_cols)
 {
-	assert(line_len < max_line);
+	assert(num_cols < max_bytes);
 	assert(max_height > *cur_height);
 
 	*start = realloc(*start, max_height * sizeof(size_t));
 	*pic = realloc(*pic, max_height * sizeof(char *));
 	*xpic = realloc(*xpic, max_height * sizeof(char *));
 
-
 	for (size_t i = *cur_height; i < max_height; i++)
 	{
 		/* Allocate memory for both pic and xpic. */
-		char *picmem = malloc(max_line * 2);
+		char *picmem = malloc(max_bytes * 2 + 2);
 
 		(*pic)[i] = picmem;
-		(*xpic)[i] = picmem + max_line;
+		(*xpic)[i] = picmem + max_bytes;
 
-		memset((*pic)[i], ' ', line_len);
-		(*pic)[i][line_len] = '\0';
+		memset((*pic)[i], ' ', num_cols);
+		(*pic)[i][num_cols] = '\0';
 	}
 
 	*cur_height = max_height;
@@ -483,7 +515,7 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 			{
 				if (ppla[j].rw == linkage->num_words-1) continue;
 				N_wall_connectors ++;
-				if (easy_match(ppla[j].lc->string, LEFT_WALL_SUPPRESS))
+				if (easy_match(connector_string(ppla[j].lc), LEFT_WALL_SUPPRESS))
 				{
 					suppressor_used = true;
 				}
@@ -504,7 +536,7 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 			if (ppla[j].rw == linkage->num_words-1)
 			{
 				N_wall_connectors ++;
-				if (easy_match(ppla[j].lc->string, RIGHT_WALL_SUPPRESS))
+				if (easy_match(connector_string(ppla[j].lc), RIGHT_WALL_SUPPRESS))
 				{
 					suppressor_used = true;
 				}
@@ -522,13 +554,22 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 	char **picture = NULL;
 	char **xpicture = NULL;
 	size_t max_height = 0;
-	size_t max_line = set_centers(linkage, center, word_offset,
+	size_t max_bytes = set_centers(linkage, center, word_offset,
 	                              print_word_0, N_words_to_print) +1;
-	unsigned int line_len = center[N_words_to_print-1]+1;
 
+	// num_cols is the total number of columns needed to display
+	// the ascii-art diagram, not counting the glyphs. It might
+	// be less, or it might be more than the total number of
+	// bytes in the UTF-8 string! Which one depends on just how
+	// much garage there might be in the string. Note also that,
+	// in general, the glyphy of the last word will stick out past
+	// the num_cols here.
+	unsigned int num_cols = center[N_words_to_print-1]+1;
+
+	if (max_bytes < num_cols) max_bytes = num_cols;
 	diagram_alloc_tmpmem(&start, &picture, &xpicture,
 	                     &max_height, HEIGHT_INC,
-	                     max_line, line_len);
+	                     max_bytes, num_cols);
 
 	top_row = 0;
 
@@ -546,6 +587,7 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 			if (!print_word_N && (ppla[j].rw == linkage->num_words-1)) continue;
 
 			/* Put it into the lowest position */
+			/* Keep in mind that cl, cr are "columns" not "bytes" */
 			cl = center[ppla[j].lw];
 			cr = center[ppla[j].rw];
 			for (row=0; row < max_height; row++)
@@ -567,7 +609,7 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 				lgdebug(+9, "Extending rows up to %d.\n", (2*row+2)+HEIGHT_INC);
 				diagram_alloc_tmpmem(&start, &picture, &xpicture,
 				                     &max_height, (2*row+2)+HEIGHT_INC,
-				                     max_line, line_len);
+				                     max_bytes, num_cols);
 			}
 			if (row > top_row) top_row = row;
 
@@ -590,17 +632,17 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 
 			/* Add direction indicator */
 			// if (DEPT_CHR == ppla[j]->lc->string[0]) { *(t-1) = '<'; }
-			if (DEPT_CHR == ppla[j].lc->string[0] &&
+			if (DEPT_CHR == connector_string(ppla[j].lc)[0] &&
 			    (t > &picture[row][cl])) { picture[row][cl+1] = '<'; }
-			if (HEAD_CHR == ppla[j].lc->string[0]) { *(t-1) = '>'; }
+			if (HEAD_CHR == connector_string(ppla[j].lc)[0]) { *(t-1) = '>'; }
 
 			/* Copy connector name; stop short if no room */
 			while ((*s != '\0') && (*t == '-')) *t++ = *s++;
 
 			/* Add direction indicator */
 			// if (DEPT_CHR == ppla[j]->rc->string[0]) { *t = '>'; }
-			if (DEPT_CHR == ppla[j].rc->string[0]) { picture[row][cr-1] = '>'; }
-			if (HEAD_CHR == ppla[j].rc->string[0]) { *t = '<'; }
+			if (DEPT_CHR == connector_string(ppla[j].rc)[0]) { picture[row][cr-1] = '>'; }
+			if (HEAD_CHR == connector_string(ppla[j].rc)[0]) { *t = '<'; }
 
 			/* The direction indicators may have clobbered these. */
 			picture[row][cl] = '+';
@@ -621,14 +663,14 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 	/* We have the link picture, now put in the words and extra "|"s */
 	t = xpicture[0];
 	if (print_word_0) k = 0; else k = 1;
-	for (; k<N_words_to_print; k++) {
-		for (i = 0; i < (size_t)word_offset[k]; i++) *t++ = ' ';
+	for (; k<N_words_to_print; k++)
+	{
+		for (i = 0; i < (size_t) word_offset[k]; i++) *t++ = ' ';
+
+		// Copy raw bytes. Adjustments for different widths,
+		// invalid utf8 characters, etc. is done later.
 		s = linkage->word[k];
-		i = 0;
-		while (*s != '\0') {
-			*t++ = *s++;
-			i++;
-		}
+		while (*s != '\0') { *t++ = *s++; }
 		*t++ = ' ';
 	}
 	*t = '\0';
@@ -667,7 +709,7 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 	/* We've built the picture, now print it out. */
 
 	/* Start locations, for each row.  These may vary, due to different
-	 * utf8 character widths. */
+	 * column-widths of utf8 glyphs. */
 	top_row_p1 = top_row + 1;
 	for (row = 0; row < top_row_p1; row++)
 		start[row] = 0;
@@ -690,33 +732,35 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 		unsigned int uwidth = 0;
 		unsigned int wwid;
 		do {
-			wwid = (c == 0)*word_offset[i] + utf8_strwidth(linkage->word[i]+c) + 1;
-			if (x_screen_width-RIGHT_MARGIN < uwidth + wwid) break;
+			wwid = ((c == 0) * word_offset[i])
+			      + utf8_strwidth(linkage->word[i]+c) + 1;
+			if (x_screen_width - RIGHT_MARGIN < uwidth + wwid) break;
 			uwidth += wwid;
 			c = 0;
 			i++;
 		} while (i < N_words_to_print);
 
-		/* The whole word doesn't fit - fit as much as possible from it. */
-		if (0 == uwidth)
-		{
-			uwidth = x_screen_width-RIGHT_MARGIN - (c == 0)*word_offset[i] - 1;
-			c += utf8_num_char(linkage->word[i]+c, uwidth);
-		}
+		if (i < N_words_to_print) {
+			/* The whole word doesn't fit - fit as much as possible from it. */
+			if (0 == uwidth)
+			{
+				uwidth = x_screen_width - RIGHT_MARGIN - (c == 0)*word_offset[i]-1;
 
-		if (NULL != pctx) /* PS junk */
-		{
-			pctx->row_starts[pctx->N_rows] = i - (!print_word_0);
-			if (i < N_words_to_print) pctx->N_rows++;
-		}
+				c += utf8_chars_in_width(linkage->word[i]+c, uwidth);
+			}
 
+			if (NULL != pctx) /* PS junk */
+			{
+				pctx->row_starts[pctx->N_rows] = i - (!print_word_0);
+				pctx->N_rows++;
+			}
+		}
 		dyn_strcat(string, "\n");
+
+		/* print each row of the picture */
 		top_row_p1 = top_row + 1;
 		for (revrs = 0; revrs < top_row_p1; revrs++)
 		{
-			/* print each row of the picture */
-			/* `blank` is used solely to detect blank lines */
-			unsigned int last_nonblank = (unsigned int)-1;
 			/*
 			 * The `glyph_width` is the width, in columns, of the printable
 			 * glyph. Chinese glyphs are almost always width two.
@@ -724,31 +768,30 @@ linkage_print_diagram_ctxt(const Linkage linkage,
 			size_t glyph_width = 0;
 
 			row = top_row - revrs;
+
+			// k is the number of bytes into the row that we previously
+			// printed. It should normally point at a valid UTF-8 char.
 			k = start[row];
 			for (j = k; (glyph_width < uwidth) && (xpicture[row][j] != '\0'); )
 			{
-				if (xpicture[row][j] != ' ') last_nonblank = j;
+				// If we don't have a glyph for this code-point,
+				// then assume the terminal will use a two-column-
+				// -wide "box font" with the hex code inside.
+				int gw = utf8_charwidth(&xpicture[row][j]);
+				if (gw < 0) gw = 2;
+				glyph_width += gw;
 
-				/* Update the printable column width */
-				glyph_width += utf8_charwidth(&xpicture[row][j]);
-
-				j += utf8_charlen(&xpicture[row][j]);
+				/* Copy exactly one multi-byte character to buf */
+				j += append_utf8_char(string, &xpicture[row][j]);
 			}
+
+			// Record exactly how much we printed, so we can
+			// resume here, if the diagram wraps.
 			start[row] = j;
 
-			if ((unsigned int)-1 != last_nonblank)
-			{
-				glyph_width = 0;
-				for (j = k; (glyph_width < uwidth) && (xpicture[row][j] != '\0'); )
-				{
-					glyph_width += utf8_charwidth(&xpicture[row][j]);
-
-					/* Copy exactly one multi-byte character to buf */
-					j += append_utf8_char(string, &xpicture[row][j]);
-					if (last_nonblank < j) break; /* Trim trailing blanks */
-				}
-				dyn_strcat(string, "\n");
-			}
+			// Hunt down and kill trailing blanks.
+			dyn_trimback(string);
+			dyn_strcat(string, "\n");
 		}
 		dyn_strcat(string, "\n");
 	}
@@ -1442,7 +1485,11 @@ void print_sentence_word_alternatives(dyn_str *s, Sentence sent, bool debugprint
 					char *info = display(sent->dict, wt);
 
 					if (NULL == info) return;
-					append_string(s, "Token \"%s\" %s", wt, info);
+					append_string(s, "Token \"%s\" ", wt);
+					/* Cannot use append_string() for "info" because it may be
+					 * a multi-MB string due to !!*
+					 * (printing all dictionary words). */
+					dyn_strcat(s, info);
 					free(info);
 				}
 				else if (word_split) append_string(s, " %s", wt);
