@@ -69,8 +69,6 @@
 #endif /* USE_VITERBI */
 
 #define DISPLAY_MAX 1024
-#define COMMENT_CHAR '%'  /* input lines beginning with this are ignored */
-#define WHITESPACE " \t\v\r\n" /* ASCII-only is sufficient here. */
 
 static int batch_errors = 0;
 static int verbosity = 0;
@@ -345,16 +343,16 @@ static const char *process_some_linkages(FILE *in, Sentence sent,
 			if ((sentence_num_valid_linkages(sent) == 1) &&
 				!copts->display_bad)
 			{
-				fprintf(stdout, "	Unique linkage, ");
+				fprintf(stdout, "\tUnique linkage, ");
 			}
 			else if (copts->display_bad &&
 			         (sentence_num_violations(sent, i) > 0))
 			{
-				fprintf(stdout, "	Linkage %d (bad), ", num_displayed+1);
+				fprintf(stdout, "\tLinkage %d (bad), ", num_displayed+1);
 			}
 			else
 			{
-				fprintf(stdout, "	Linkage %d, ", num_displayed+1);
+				fprintf(stdout, "\tLinkage %d, ", num_displayed+1);
 			}
 
 			corpus_cost = linkage_corpus_cost(linkage);
@@ -453,14 +451,16 @@ static void batch_process_some_linkages(Label label,
 	}
 }
 
-static bool special_command(char *input_string, Command_Options* copts, Dictionary dict)
+/**
+ * If input_string is !command, try to issue it.
+ */
+
+static char special_command(char *input_string, Command_Options* copts, Dictionary dict)
 {
-	if (input_string[0] == COMMENT_CHAR) return true;
-	if (input_string[0] == '!') {
-		issue_special_command(input_string+1, copts, dict);
-		return true;
-	}
-	return false;
+	if (input_string[0] == COMMENT_CHAR) return 'c';
+	if (input_string[0] == '!')
+		return issue_special_command(input_string+1, copts, dict);
+	return 'n';
 }
 
 static Label strip_off_label(char * input_string)
@@ -492,7 +492,7 @@ static void setup_panic_parse_options(Parse_Options opts)
 	parse_options_set_spell_guess(opts, 0);
 }
 
-static void print_usage(char *str)
+static void print_usage(char *str, int exit_value)
 {
 	Command_Options *copts;
 	fprintf(stderr,
@@ -503,7 +503,7 @@ static void print_usage(char *str)
 	fprintf(stderr, "\nSpecial commands are:\n");
 	copts = command_options_create();
 	issue_special_command("var", copts, NULL);
-	exit(-1);
+	exit(exit_value);
 }
 
 /**
@@ -525,7 +525,7 @@ static void check_winsize(Command_Options* copts)
 	/* Calculate the size of the console window. */
 	if (GetConsoleScreenBufferInfo(console, &info) == 0) goto fail;
 
-	copts->screen_width = info.srWindow.Right - info.srWindow.Left + 1;
+	copts->screen_width = (size_t)(info.srWindow.Right - info.srWindow.Left + 1);
 	return;
 
 fail:
@@ -598,11 +598,17 @@ int main(int argc, char * argv[])
 		i++;
 	}
 
+	if ((i < argc) && strcmp("--help", argv[i]) == 0)
+	{
+		print_usage(argv[0], 0);
+	}
+
 	for (; i<argc; i++)
 	{
 		if (argv[i][0] == '-' && strcmp("--version", argv[i]) == 0)
 		{
 			printf("Version: %s\n", linkgrammar_get_version());
+			printf("%s\n", linkgrammar_get_configuration());
 			exit(0);
 		}
 	}
@@ -625,14 +631,21 @@ int main(int argc, char * argv[])
 	parse_options_set_short_length(opts, 16);
 	parse_options_set_islands_ok(opts, false);
 
+	save_default_opts(copts); /* Options so far are the defaults */
+
 	/* Process command line variable-setting commands (only) */
 	for (i = 1; i < argc; i++)
 	{
 		if (argv[i][0] == '-')
 		{
 			const char *var = argv[i] + ((argv[i][1] != '-') ? 1 : 2);
-			if ((var[0] != '!') && issue_special_command(var, copts, NULL))
-				print_usage(argv[0]);
+			if ((var[0] != '!') && (0 > issue_special_command(var, copts, NULL)))
+				print_usage(argv[0], -1);
+		}
+		else if (i != 1)
+		{
+			prt_error("Fatal error: Unknown argument '%s'.\n", argv[i]);
+			print_usage(argv[0], -1);
 		}
 	}
 
@@ -665,7 +678,7 @@ int main(int argc, char * argv[])
 		if ((argv[i][0] == '-') && (argv[i][1] == '!'))
 		{
 			if (issue_special_command(argv[i]+1, copts, dict))
-				print_usage(argv[0]);
+				print_usage(argv[0], -1);
 		}
 	}
 
@@ -712,16 +725,28 @@ int main(int argc, char * argv[])
 			*p = '\0';
 		}
 
-		if ((strcmp(input_string, "!quit") == 0) ||
-		    (strcmp(input_string, "!exit") == 0)) break;
+		/* If the input string is just whitespace, then ignore it. */
+		if (strspn(input_string, WHITESPACE) == strlen(input_string))
+			continue;
+
+		char command = special_command(input_string, copts, dict);
+		if ('e' == command) break;    /* It was an exit command */
+		if ('c' == command) continue; /* It was another command */
+		if (-1 == command) continue;  /* It was a bad command */
 
 		/* We have to handle the !file command inline; its too hairy
 		 * otherwise ... */
-		if (strncmp(input_string, "!file", 5) == 0)
+		if ('f' == command)
 		{
-			char * filename = &input_string[6];
+			char * filename = &input_string[strcspn(input_string, WHITESPACE)] + 1;
 			int fnlen = strlen(filename);
 
+			if (0 == fnlen)
+			{
+				prt_error("Error: Missing file name argument\n");
+				continue;
+			}
+
 			if ('\n' == filename[fnlen-1]) filename[fnlen-1] = '\0';
 
 			struct stat statbuf;
@@ -744,11 +769,6 @@ int main(int argc, char * argv[])
 			continue;
 		}
 
-		/* If the input string is just whitespace, then ignore it. */
-		if (strspn(input_string, WHITESPACE) == strlen(input_string))
-			continue;
-
-		if (special_command(input_string, copts, dict)) continue;
 
 		if (!copts->batch_mode) batch_in_progress = false;
 		if ('\0' != test[0])
@@ -809,6 +829,32 @@ int main(int argc, char * argv[])
 				sent = NULL;
 				continue;
 			}
+
+			if (0 != copts->display_wordgraph)
+			{
+				const char *wg_display_flags = ""; /* default flags */
+				switch (copts->display_wordgraph)
+				{
+					case 1:     /* default flags */
+						break;
+					case 2:     /* subgraphs with a legend */
+						wg_display_flags = "sl";
+						break;
+					case 3:
+						{
+							/* Use esoteric flags from the test user variable. */
+							const char wg[] = ",wg";
+							const char *s = strstr(test, wg);
+							if (NULL != s) wg_display_flags = s+2;
+						}
+						break;
+					default:
+						prt_error("Warning: wordgraph=%d: Unknown value, using 1\n",
+						          copts->display_wordgraph);
+						copts->display_wordgraph = 1;
+				}
+				sentence_display_wordgraph(sent, wg_display_flags);
+			}
 #if 0
 			/* Try again, this time omitting the requirement for
 			 * definite articles, etc. This should allow for the parsing
